Search.setIndex({docnames:["index","quick_start"],envversion:{"sphinx.domains.c":1,"sphinx.domains.changeset":1,"sphinx.domains.cpp":1,"sphinx.domains.javascript":1,"sphinx.domains.math":2,"sphinx.domains.python":1,"sphinx.domains.rst":1,"sphinx.domains.std":1,"sphinx.ext.viewcode":1,sphinx:55},filenames:["index.rst","quick_start.rst"],objects:{},objnames:{},objtypes:{},terms:{content:0,etc:1,instal:1,quick:0,requir:1,start:0},titles:["Django Lanthanum Documentation","Quick Start"],titleterms:{django:0,document:0,lanthanum:0,quick:1,start:1}})